<?php

namespace Drupal\crossword\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'crossword_screenreader' formatter.
 *
 * @FieldFormatter(
 *   id = "crossword_screenreader",
 *   label = @Translation("Crossword Puzzle (screenreader)"),
 *   weight = "2",
 *   field_types = {
 *     "crossword"
 *   }
 * )
 */
class CrosswordFormatterScreenreader extends CrosswordFormatter {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $options['print'] = TRUE;
    $options['details'] = [
      'title_tag' => 'h1',
      'author_tag' => 'h2',
      'notepad_tag' => 'p',
    ];
    $options['buttons']['buttons'] = [
      'solution' => 'solution',
      'clear' => 'clear',
    ];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $form = parent::settingsForm($form, $form_state);
    unset($form['print']);

    // Restrict the available buttons.
    $form['buttons']['buttons']['#options'] = [
      'solution' => $this->t('Solution'),
      'clear' => $this->t('Clear'),
    ];

    // Remove checkboxes.
    unset($form['errors']);
    unset($form['references']);
    unset($form['rebus']);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $file) {

      $data = \Drupal::service('crossword.data_service')->getData($file);

      $elements[$delta] = [
        '#theme' => 'crossword',
        '#screenreader' => TRUE,
        '#content' => [
          'title' => $this->getTitle($data),
          'author' => $this->getAuthor($data),
          'notepad' => $this->getNotepad($data),
          'grid' => $this->getGrid($data),
          'across' => $this->getAcross($data),
          'down' => $this->getDown($data),
          'controls' => $this->getControls($file),
        ],
        '#attached' => [
          'library' => [
            'crossword/crossword.screenreader',
            $this->getSetting('print') ? 'crossword/crossword.print' : '',
          ],
          'drupalSettings' => [
            'crossword' => [
              'data' => $data,
              'selector' => '.crossword',
            ],
          ],
        ],
        '#cache' => [
          'tags' => $file->getCacheTags(),
        ],
      ];
      // We don't need or want the rebus checkbox.
      unset($elements[$delta]['#content']['controls']['#content']['rebus']);
    }
    return $elements;
  }

}
